#!/usr/bin/env python

#tinkered commands:
#$google [site] [-num]

#Python libraries (preinstalled)
import datetime
import ftplib
import hashlib
import os
import random
import sys
import time
import urllib

#Twisted libraries (dependency- you must install zope.interface as well)
from twisted.words.protocols import irc
from twisted.internet import protocol
from twisted.internet import reactor

#calcpkg ticalc.org stuff
import calcpkg

#channel list for this bot to run on
channels = ['#cemetech3', '#omnimaga3']

class LogBot(irc.IRCClient):
	def _get_nickname(self):
		return self.factory.nickname
	nickname = property(_get_nickname)

	operators = []

	def signedOn(self):
		global channels
		self.join(self.factory.channel)
		for chan in channels:
			self.join(chan)
		
	def joined(self, channel):
		print "*** Joined " + channel
		
	def privmsg(self, user, channel, msg):
		#First, if this is a message from a freenode server, DO put it in the logs, but don't parse it- just quit
		if '.net' in user:
			print msg
			return

		#Get rid of the user's hostmask
		if not '#' in user:
			shortuser = user[:user.find('!')]

		if channel == self.nickname:
			channel = shortuser

		#IRC commands in this bot
		if msg[0] == '!':
			#calcpkg commands from IRC
			if "calcpkg" in msg[1:]:
				if "calcpkg count" in msg[1:]:
					self.msg(channel, calcpkg.ticalcCount(msg[15:]))
				elif "calcpkg list" in msg[1:]:
					data = calcpkg.ticalcSearchIndex(msg[14:])
					newpaste = structureSearchOutput("File Path/Category:", "File Name:") + "\n"
					newpaste += "======================================================"
					newpaste += "=================================================================" + "\n"
					for datum in data:
						newpaste += structureSearchOutput(datum[0], datum[1]) + "\n"
					self.msg(channel, pastebin(newpaste, False))
				if "calcpkg update" == msg[1:]:
					if shortuser in self.operators:
						calcpkg.ticalcUpdatePackageLists(False)
						self.msg(channel, "Finished updating package lists")
					else:
						self.msg(channel, "You must be an operator to perform this command")
				if "calcpkg help" == msg[1:]:
					self.msg(channel, "  $calcpkg count [text]: outputs the number of times [text] appears in the indexes")
					self.msg(channel, "  $calcpkg list [text]: performs a full search of the indexes and pastebins the results")
					self.msg(channel, "  $calcpkg update: updates the indexes used by calcpkg from ticalc.org, requires op")

			#nick command, to change bot nickname remotely
			elif "nick" in msg[1:]:
				if shortuser in self.operators:
					try:
						newnick = msg[6:]
					except:
						self.msg(channel, "Invalid use of !nick command")
						return
					self.setNick(newnick)
				else:
					self.msg(channel, "You must be an operator to perform this command")

	def logSay(self, msg):
		"""Both log and say a message from the bot"""
		channel = self.factory.channel
		self.say(channel, msg)
		
	def bashSay(self, channel, bashquote):
		"""Says each line of a bash.org quote"""
		numsaid = 0
		while len(bashquote) > 0:
			linelen = bashquote.find('\n')
			linequote = bashquote[:linelen]
			if linequote == "":
				bashquote = bashquote[1:]
				continue
			self.msg(channel, linequote)
			bashquote = bashquote[linelen + 1:]
			if linelen == 0:
				bashquote = ""
			numsaid += 1
			if numsaid == 8:
				time.sleep(1)
		
	#Update the self.operators list
	def userLeft(self, user, channel):
              	if user in self.operators:
			self.operators.remove(user)
		
	def userQuit(self, user, quitMessage):
              	if user in self.operators:
			self.operators.remove(user)
		
	def userKicked(self, kickee, channel, kicker, message):
              	if kickee in self.operators:
			self.operators.remove(user)
		
	def userRenamed(self, oldname, newname):
              	if oldname in self.operators:
			self.operators.remove(oldname)
			self.operators.append(newname)
		
	def modeChanged(self, user, channel, set, modes, args):
		#Get the + or - string
		strset = '-'
		if set:
			strset = '+'
			
		#Get the shortuser (cut the hostmask)
		shortlen = user.find('!')
		shortuser = user[:user.find('!')]
		if shortlen == -1:
			shortuser = user
		
		#Determine if it's being set on a user or not
		try:
			setuser = args[0]
		except:
			setuser = ''
		
		#If this is operator mode- o- add or remove operator status to operator array
		if ('o' in modes and setuser != ""):
			if set:
				if not user in self.operators:
					self.operators.append(setuser)
			else:
				try:
					self.operators.remove(setuser)
				except IndexError:
					pass

class LogBotFactory(protocol.ClientFactory):
	protocol = LogBot

	#Name temporary...
	def __init__(self, channel, nickname='Hex'):
		self.channel = channel
		self.nickname = nickname

	def clientConnectionLost(self, connector, reason):
		print "Lost connection (%s), reconnecting." % (reason,)
		connector.connect()

	def clientConnectionFailed(self, connector, reason):
		print "Could not connect: %s" % (reason,)
	
def logprint(message):
	"""Function to print activity to the command line and then log it to a file on my system."""

	#Output the data to the console
	now = time.localtime()
	hour = fixTime(str(now[3]))
	minute = fixTime(str(now[4]))
	second = fixTime(str(now[5]))
	fullmsg = '[' + fixTime(hour) + ':' + fixTime(minute) + ':' + fixTime(second) + '] ' + message
	print fullmsg

def fixTime(string):
	"""Utility to fix problems for a 'time' string."""
	if len(string) == 1:
		string = '0' + string
	return string

def structureSearchOutput(fileurl, filename):
	"""Formats the output of a calcpkg list command"""
	output = fileurl
	if len(output) > 60:
		output += " " + fileurl
	else:
		while len(output) <= 60:
			output += " "
		output += filename
	return output

def pastebin(sourcetext, python):
	"""Function that uses Pastebin API to make the source publically available"""
	
	#Build the pasteargs dictionary containing all the data to pastebin
	pasteargs = {'paste_code':sourcetext}
	pasteargs['paste_private'] = 0
	pasteargs['paste_expire_date'] = '1H'
	pasteargs['paste_format'] = 'text'
	if python:
		pasteargs['paste_format'] = 'python'
	
	#Make a request to the Pastebin API
	pasteapi = urllib.urlopen('http://pastebin.com/api_public.php', urllib.urlencode(pasteargs))
	try:
		response = pasteapi.read()
	finally:
		pasteapi.close()
	del pasteapi
	
	#Return a string of either the URL or an error message
	pastesite = 'http://pastebin.com/'
	if not response.startswith(pastesite):
		return "Pastebin failed [" + response + "]"
	return response
	
def main():
	"""Function to handle user interput."""	
	#Try to get the channel name
	try:
		chan = sys.argv[1]
	except IndexError:
		print ""
		print "Please enter a channel name"
		print "Leave out the #, so: ti, not #ti"
		print "By default, the bot will join #cemetech and #omnimaga on top of what you enter here"
		print "(unless you edit the script otherwise)"
		chan = raw_input("Channel Name: ")
	
	#Try to get the network name
	try:
		network = sys.argv[2]
	except IndexError:
		print ""
		print "Please enter a network name"
		print "For example, irc.freenode.net"
		network = raw_input("Network Name: ")

	#Define the global channel
	global gchannel
	gchannel = '#'+chan
		
	#Run the IRC bot
	print ""
	reactor.connectTCP(network, 6667, LogBotFactory(gchannel))
	reactor.run()
	
if __name__ == "__main__":
	main()
